#! /usr/bin/env -S ts-node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const util_1 = __importDefault(require("util"));
const ansi_colors_1 = __importDefault(require("ansi-colors"));
const inflected_1 = require("inflected");
if (process.env.NODE_ENV === "test") {
    ansi_colors_1.default.enabled = false;
}
const CLI = {
    default(commandHandler) {
        !process.argv[2] ? commandHandler() : null;
    },
    command(commandName, commandHandler) {
        if (Array.isArray(commandName)) {
            return commandName.includes(process.argv[2]) ? commandHandler() : null;
        }
        commandName === process.argv[2] ? commandHandler() : null;
    }
};
CLI.default(printCommands);
CLI.command(["help", "h"], printCommands);
CLI.command(["init", "new"], async () => {
    let memServerDirectory = await getMemServerDirectory();
    let boilerplateDirectory = `${__dirname}/../memserver-boilerplate`;
    if (!memServerDirectory) {
        memServerDirectory = "./memserver";
        await fs_extra_1.default.mkdir(memServerDirectory);
    }
    if (!(await fs_extra_1.default.pathExists(`${memServerDirectory}/index.ts`))) {
        const indexCode = await fs_extra_1.default.readFile(`${boilerplateDirectory}/index.ts`);
        await fs_extra_1.default.writeFile(`${memServerDirectory}/index.ts`, indexCode);
        console.log(ansi_colors_1.default.cyan("[Memserver CLI] /memserver/index.ts created"));
    }
    if (!(await fs_extra_1.default.pathExists(`${memServerDirectory}/routes.ts`))) {
        const routesCode = await fs_extra_1.default.readFile(`${boilerplateDirectory}/routes.ts`);
        await fs_extra_1.default.writeFile(`${memServerDirectory}/routes.ts`, routesCode);
        console.log(ansi_colors_1.default.cyan("[Memserver CLI] /memserver/routes.ts created"));
    }
    if (!(await fs_extra_1.default.pathExists(`${memServerDirectory}/initializer.ts`))) {
        const initializerCode = await fs_extra_1.default.readFile(`${boilerplateDirectory}/initializer.ts`);
        await fs_extra_1.default.writeFile(`${memServerDirectory}/initializer.ts`, initializerCode);
        console.log(ansi_colors_1.default.cyan("[Memserver CLI] /memserver/initializer.ts created"));
    }
    await createFixtureAndModelFoldersIfNeeded(memServerDirectory);
});
CLI.command(["generate", "g"], async () => {
    const memServerDirectory = await getMemServerDirectory();
    const generationType = process.argv[3];
    const modelName = process.argv[4] ? process.argv[4].toLocaleLowerCase() : null;
    if (!memServerDirectory) {
        return console.log(ansi_colors_1.default.red("[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?"));
    }
    else if (!generationType) {
        return console.log(ansi_colors_1.default.red("[Memserver CLI] generate should be either $ memserver g model [modelName] or $ memserver g fixtures"));
    }
    else if (generationType === "model") {
        return await generateModel(modelName, memServerDirectory);
    }
    else if (generationType === "fixtures") {
        return await generateFixtures(modelName, memServerDirectory);
    }
    console.log(ansi_colors_1.default.red(`[Memserver CLI] $ memserver ${process.argv[2]} ${process.argv[3]} ${process.argv[4] ||
        ""} does not exists, available commands:`));
    return printCommands();
});
CLI.command(["console", "c"], async () => await openConsole());
CLI.command(["version", "v"], async () => {
    console.log(ansi_colors_1.default.cyan("[Memserver CLI]"), JSON.parse((await fs_extra_1.default.readFile(`${__dirname}/../package.json`)).toString()).version);
});
async function printCommands() {
    const config = JSON.parse((await fs_extra_1.default.readFile(`${__dirname}/../package.json`)).toString());
    const highlight = (text) => ansi_colors_1.default.bold.cyan(text);
    console.log(`${highlight("[Memserver CLI v" + config.version + "] Usage:")} memserver ${ansi_colors_1.default.yellow("<command (Default: help)>")}

memserver init | new                    # Sets up the initial memserver folder structure
memserver generate model ${ansi_colors_1.default.yellow("[ModelName]")}    # Generates the initial files for a MemServer Model ${ansi_colors_1.default.cyan('[alias: "memserver g model"]')}
memserver generate fixtures             # Outputs your initial MemServer state as pure javascript fixture files
memserver generate fixtures ${ansi_colors_1.default.yellow("[ModelName]")} # Outputs your initial MemServer state for certain model as pure javascript fixture
memserver console                       # Starts a MemServer console in node.js ${ansi_colors_1.default.cyan('[alias: "memserver c"]')}`);
}
async function generateModel(modelName, memServerDirectory) {
    if (!modelName) {
        return console.log(ansi_colors_1.default.red("[Memserver CLI] Please put a modelName to the memserver generate. Example: $ memserver generate model user"));
    }
    await createFixtureAndModelFoldersIfNeeded(memServerDirectory);
    const modelFileName = inflected_1.dasherize(inflected_1.singularize(modelName));
    const fixtureFileName = inflected_1.dasherize(inflected_1.pluralize(modelName));
    if (!(await fs_extra_1.default.pathExists(`${memServerDirectory}/models/${modelFileName}.ts`))) {
        const classModelName = inflected_1.classify(modelName);
        await fs_extra_1.default.writeFile(`${memServerDirectory}/models/${modelFileName}.ts`, `import Model from 'memserver/model';

export default class ${classModelName} extends Model {
  constructor() {
    super();
  }
}`);
        console.log(ansi_colors_1.default.cyan(`[Memserver CLI] /memserver/models/${modelFileName}.ts created`));
    }
    if (!(await fs_extra_1.default.pathExists(`${memServerDirectory}/fixtures/${fixtureFileName}.ts`))) {
        await fs_extra_1.default.writeFile(`${memServerDirectory}/fixtures/${fixtureFileName}.ts`, `export default [
];`);
        console.log(ansi_colors_1.default.cyan(`[Memserver CLI] /memserver/fixtures/${fixtureFileName}.ts created`));
    }
}
// TODO: this has to change MemServer.start() will change
async function generateFixtures(modelName, memServerDirectory) {
    const Model = (await Promise.resolve().then(() => __importStar(require("./model")))).default;
    const startMemserver = (await Promise.resolve().then(() => __importStar(require("./index")))).default;
    const Server = await startMemserver();
    const ModelDefinitions = await getModelDefinitions(memServerDirectory);
    const targetModels = modelName
        ? [inflected_1.classify(inflected_1.singularize(modelName))]
        : Object.keys(ModelDefinitions);
    targetModels.forEach(async (Model) => {
        const sortedState = ModelDefinitions[Model].findAll().sort(sortFunction);
        const arrayOfRecords = util_1.default.inspect(sortedState, {
            depth: null,
            maxArrayLength: null
        });
        const targetFileName = inflected_1.pluralize(inflected_1.dasherize(inflected_1.underscore(Model)));
        const fileRelativePath = `/fixtures/${targetFileName}.ts`;
        const fileAbsolutePath = `${memServerDirectory}${fileRelativePath}`;
        if (await fs_extra_1.default.pathExists(fileAbsolutePath)) {
            const previousModels = (await Promise.resolve().then(() => __importStar(require(fileAbsolutePath)))).default;
            if (JSON.stringify(previousModels.sort(sortFunction)) === JSON.stringify(sortedState)) {
                return;
            }
        }
        await fs_extra_1.default.writeFile(fileAbsolutePath, `export default ${arrayOfRecords};`, () => {
            console.log(ansi_colors_1.default.yellow(`[MemServer] data written to ${fileRelativePath}`));
        });
    });
}
async function createFixtureAndModelFoldersIfNeeded(memServerDirectory) {
    let boilerplateDirectory = `${__dirname}/../memserver-boilerplate`;
    if (!(await fs_extra_1.default.pathExists(`${memServerDirectory}/fixtures`))) {
        await fs_extra_1.default.mkdir(`${memServerDirectory}/fixtures`);
        await fs_extra_1.default.copy(`${boilerplateDirectory}/fixtures`, `${memServerDirectory}/fixtures`),
            console.log(ansi_colors_1.default.cyan("[Memserver CLI] /memserver/fixtures folder created"));
    }
    if (!(await fs_extra_1.default.pathExists(`${memServerDirectory}/models`))) {
        await fs_extra_1.default.mkdir(`${memServerDirectory}/models`);
        await fs_extra_1.default.copy(`${boilerplateDirectory}/models`, `${memServerDirectory}/models`);
        console.log(ansi_colors_1.default.cyan("[Memserver CLI] /memserver/models folder created"));
    }
}
async function openConsole() {
    if (process.cwd().includes("memserver")) {
        throw new Error(ansi_colors_1.default.red("[Memserver CLI] You are in the memserver directory, go to the root of your project to start memserver console."));
    }
    const MemServer = await (await Promise.resolve().then(() => __importStar(require("./index")))).default();
    const repl = (await Promise.resolve().then(() => __importStar(require("repl")))).default;
    console.log(ansi_colors_1.default.cyan("[Memserver CLI]"), "Started MemServer node.js console - check window.Memserver, window.MemServer and import/use your models ;)");
    repl.start("> ");
}
async function getMemServerDirectory() {
    const cwd = process.cwd();
    const folders = cwd.split("/");
    const memServerIndex = folders.findIndex((path) => path === "memserver");
    if (memServerIndex !== -1) {
        return folders.slice(0, memServerIndex + 1).join("/");
    }
    else if ((await fs_extra_1.default.readdir(".")).includes("memserver")) {
        return `${cwd}/memserver`;
    }
}
function sortFunction(a, b) {
    if (a.id > b.id) {
        return 1;
    }
    else if (a.id < b.id) {
        return -1;
    }
    return 0;
}
async function getModelDefinitions(memServerDirectory) {
    const models = await fs_extra_1.default.readdir(`${memServerDirectory}/models`);
    const modelDefinitions = await Promise.all(models.map(async (modelPath) => {
        return (await Promise.resolve().then(() => __importStar(require(`${memServerDirectory}/models/${modelPath}`)))).default;
    }));
    return modelDefinitions.reduce((result, modelDefinition) => {
        return Object.assign(result, { [modelDefinition.name]: modelDefinition });
    }, {});
}
